/**
 * Created by AlexanderC on 6/2/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Factory = undefined;

var _Resource = require('./Resource');

var _Collection = require('./Collection');

var _Action = require('./Action');

var _Policy = require('./Policy');

var _Statement = require('./Statement');

/**
 *  AWS IAM factory
 */
class Factory {
  /**
   * @param {Function|*} Prototype
   * @param {Array} args
   * @returns {Object}
   */
  static create(Prototype, ...args) {
    Prototype = Factory._assurePrototype(Prototype);

    return new Prototype(...args);
  }

  /**
   * @param {Function|*} Prototype
   * @returns {Collection}
   */
  static createCollection(Prototype) {
    Prototype = Factory._assurePrototype(Prototype);

    return new _Collection.Collection(Prototype);
  }

  /**
   * @param {Function|*} Prototype
   * @returns {Function}
   * @private
   */
  static _assurePrototype(Prototype) {
    if (typeof Prototype === 'string') {
      Prototype = Factory[Prototype.toUpperCase()];
    }

    return Prototype;
  }

  /**
   * @returns {Policy}
   * @constructor
   */
  static get POLICY() {
    return _Policy.Policy;
  }

  /**
   * @returns {Action}
   * @constructor
   */
  static get RESOURCE() {
    return _Resource.Resource;
  }

  /**
   * @returns {Resource}
   * @constructor
   */
  static get ACTION() {
    return _Action.Action;
  }

  /**
   * @returns {Statement}
   * @constructor
   */
  static get STATEMENT() {
    return _Statement.Statement;
  }
}
exports.Factory = Factory;