/**
 * Created by AlexanderC on 5/27/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Resource = undefined;

var _Extractable = require('./Extractable');

var _Region = require('../Region');

var _Service = require('../Service');

var _InvalidArgumentException = require('../../Exception/InvalidArgumentException');

var _InvalidArnException = require('./Exception/InvalidArnException');

/**
 * IAM statement resource
 */
class Resource extends _Extractable.Extractable {
  constructor(service = _Service.Service.ANY, region = '', accountId = '', descriptor = '') {
    super();

    this._service = null;
    this._region = null;
    this._accountId = accountId;
    this._descriptor = descriptor;

    this.service = service;
    this.region = region;

    this._any = false;
  }

  /**
   * @returns {Resource}
   */
  any() {
    this._any = true;
    return this;
  }

  /**
   * @param {String} identifier
   */
  set accountId(identifier) {
    this._accountId = identifier;
  }

  /**
   * @returns {String}
   */
  get accountId() {
    return this._accountId;
  }

  /**
   * @param {String} descriptor
   */
  set descriptor(descriptor) {
    this._descriptor = descriptor;
  }

  /**
   * @returns {String}
   */
  get descriptor() {
    return this._descriptor;
  }

  /**
   * @param {String} name
   */
  set region(name) {
    if (!_Region.Region.exists(name)) {
      throw new _InvalidArgumentException.InvalidArgumentException(name, _Region.Region);
    }

    this._region = name;
  }

  /**
   * @returns {String}
   */
  get region() {
    return this._region;
  }

  /**
   * @param {String} name
   */
  set service(name) {
    if (!_Service.Service.exists(name)) {
      throw new _InvalidArgumentException.InvalidArgumentException(name, _Service.Service);
    }

    this._service = name;
  }

  /**
   * @returns {String}
   */
  get service() {
    return this._service;
  }

  /**
   * @see - http://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html
   *
   * @param {string} arn
   * @returns {Resource}
   */
  updateFromArn(arn) {
    let arnParts = arn.split(':');

    if (arnParts.length < 6) {
      throw new _InvalidArnException.InvalidArnException(arn);
    }

    this.service = arnParts[2];
    this.region = arnParts[3];
    this.accountId = arnParts[4];
    this.descriptor = arnParts.slice(5).join(':');

    return this;
  }

  /**
   * @returns {String}
   */
  extract() {
    if (this._any) {
      return '*';
    }

    let service = this._service;
    let region = this._region;
    let accountId = this._accountId;
    let descriptor = this._descriptor;

    return `arn:aws:${service}:${region}:${accountId}:${descriptor}`;
  }
}
exports.Resource = Resource;