/**
 * Created by AlexanderC on 1/21/16.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Sandbox = undefined;

var _domain = require('domain');

var _domain2 = _interopRequireDefault(_domain);

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Sandbox {
  /**
   * @param {Function} func
   */
  constructor(func) {
    this._func = func;
    this._onFail = Sandbox.ON_FAIL_CB;
  }

  /**
   * @returns {Function}
   */
  get func() {
    return this._func;
  }

  /**
   * @returns {Sandbox}
   */
  run(...args) {
    let execDomain = _domain2.default.create();
    let failCbTriggered = false;

    let failCb = error => {
      if (failCbTriggered) {
        return;
      }

      failCbTriggered = true;

      try {
        execDomain.exit();
      } catch (e) {/* silent fail */}

      setImmediate(() => {
        this._onFail(error);
      });
    };

    execDomain.once('error', failCb);

    // lambda predefined listeners forces "Process exited ..." error
    _process2.default.removeAllListeners('uncaughtException');
    _process2.default.removeAllListeners('unhandledRejection');
    _process2.default.on('uncaughtException', failCb);
    _process2.default.on('unhandledRejection', failCb);

    try {
      execDomain.run(this._func, ...args);
    } catch (error) {
      failCb(error);
    }

    return this;
  }

  /**
   * @param {Function} cb
   * @returns {Sandbox}
   */
  fail(cb) {
    this._onFail = cb;

    return this;
  }

  /**
   * @returns {Function}
   */
  static get ON_FAIL_CB() {
    return error => {
      console.error(error);
    };
  }
}
exports.Sandbox = Sandbox;